package org.signwriting;

import com.dtai.gwt.BorderGadget;
import com.dtai.gwt.ButtonGadget;
import com.dtai.gwt.GadgetDialog;
import com.dtai.gwt.LabelGadget;
import com.dtai.gwt.PanelGadget;
import com.dtai.gwt.CheckboxGadget;
import com.dtai.gwt.CheckboxGadgetGroup;
import com.dtai.gwt.GadgetBorderLayout;
import com.dtai.gwt.GadgetFlowLayout;
import com.dtai.gwt.GadgetGridLayout;
import com.dtai.gwt.TextFieldGadget;
import com.dtai.gwt.TextAreaGadget;
import com.dtai.gui.StdDialog;
import java11.awt.event.ActionEvent;
import java11.awt.event.ActionListener;
import java11.awt.event.KeyEvent;
import java11.awt.event.KeyAdapter;

public class DictionaryDialog extends StdDialog {

    private static final String dialogPrefix = "DictionaryDialog.";
    
    private SignWriter signwriter;
    private TextFieldGadget textField;
    private TextAreaGadget signField;
    private Dictionary dictionary = Dictionary.getDictionary();
    private int entryNum = -1;
    private ButtonGadget insertButton;    
    //private CountryComboBox countries;

    public DictionaryDialog(SignWriter signwriter) {
        super(signwriter.getFrame(), "Dictionary", false);
        this.signwriter = signwriter;
        new Buttons();

        PanelGadget panel = getBody();
        panel.setLayout(new GadgetBorderLayout());
        getMainPanel().addKeyListener(new KeyListener());

        textField = new TextFieldGadget(30);
        
        signField = new TextAreaGadget(4,20);
        signField.setFont(SignFont.large);
        signField.setEditable(false);

        panel.add("North", textField);
        panel.add("Center", signField);
        
        /*Perhaps not the most visually appealing gui...
        BorderGadget countrySel = new BorderGadget();
        //PanelGadget countrySel = new PanelGadget();
        countrySel.setBorderType(BorderGadget.ETCHED_IN);
        countrySel.setBorderThickness(2);
        //countrySel.setLayout(new GadgetGridLayout(0,2));
        countrySel.setLayout(new GadgetFlowLayout());
        panel.add("East", countrySel);
        
        countrySel.add(new LabelGadget("Country:"));
        countrySel.add(countries = new CountryComboBox());*/
    }
    
    private void showEntry() {
        if (entryNum >= 0) {
            DictionaryEntry entry = dictionary.getEntry(entryNum);
            signField.setText(entry.makeSignLabel());
            textField.setText(entry.getText());
        } else {
            com.dtai.util.ShowUser.warning("No match found."); // seems like this
                                                           // should not happen
        }
    }
    
    private void lookup() {
        entryNum = dictionary.lookup(textField.getText());
        showEntry();
        insertButton.requestFocus();
    }
    
    public void show() {
        super.show();
        textField.selectAll();
        textField.requestFocus();
    }

    private void previous() {
        if (entryNum > 0) {
            entryNum--;
            showEntry();
        } else {
            com.dtai.util.ShowUser.warning("This is the first entry in the dictionary.");
        }
    }

    private void next() {
        if (entryNum < dictionary.getNumEntries()-1) {
            entryNum++;
            showEntry();
        } else {
            com.dtai.util.ShowUser.warning("This is the last entry in the dictionary.");
        }
    }
    
    private void insert() {
        textField.selectAll();
        textField.requestFocus();
        String signText = signField.getText();
        signwriter.insert(signText);
    }
    
    private class Buttons { // Symantec Java compiler bug forced me to
                    // create this bogus class "Buttons"

        public Buttons() {

            addButton("Lookup",
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        lookup();
                    }
                }
            ).setAsDefault();

            addButton("Previous",
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        previous();
                    }
                }
            );

            addButton("Next",
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        next();
                    }
                }
            );

            insertButton = addButton("Insert",
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        insert();
                    }
                }
            );

            addButton("Cancel",
                new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        hide();
                    }
                }
            );
        }

        private ButtonGadget addButton(String label, ActionListener listener) {
            ButtonGadget button = new ButtonGadget();
            String resourceKey = dialogPrefix+"Buttons."+Resources.labelToKey(label);
            Resources.apply(resourceKey, label, button);
            addButtonGadget(button);
            button.addActionListener(listener);
            return button;
        }
    }
    
    private class KeyListener extends KeyAdapter {
        public void keyPressed(KeyEvent e) {
            //if (countries.getList().getItemIndex(e.getSource().toString()) == -1) {
                //we did NOT press on any of the items in the countries list
                //which doesn't do anything right now anyway!!  Someday we will
                //add a listener to that combobox, and change dictionaries, or
                //something...
                //if PGUP or UP or Alt-p or Alt-P
                if ( e.getKeyCode() == KeyEvent.PGUP ||
                     e.getKeyCode() == KeyEvent.UP || 
                    ((e.getModifiers() & e.ALT_MASK) == e.ALT_MASK &&
                    ((e.getKeyChar() == 'P' || e.getKeyChar() == 'p')))) {
                    previous();
                }
                //if PGDN or DN or Alt-n or Alt-N
                else if (e.getKeyCode() == KeyEvent.PGDN ||
                    e.getKeyCode() == KeyEvent.DOWN ||
                   ((e.getModifiers() & e.ALT_MASK) == e.ALT_MASK &&
                   ((e.getKeyChar() == 'N' || e.getKeyChar() == 'n')))) {
                    next();
                }
            //}
        }
    }
}
