package org.signwriting;

public class DictionaryEntry {
    private static final int DRECEOL = 0xffff;
    
    private int recnum;
    private String text;
    private String upperText;
    private byte signData[];
    private DictionaryFile dictionary;
    
    public DictionaryEntry(DictionaryFile dictionary, int recnum, String key) {
        this.dictionary = dictionary;
        this.recnum = recnum;
        setText(key);
    }
    
    public void loadEntry() {
        byte[] textArray = null;
        do {
            DictionaryRecord rec = dictionary.getRecord(recnum);
            int reclen = rec.getLen();
            if (reclen > 0) {
                if (signData == null) {
                    signData = new byte[reclen];
                    System.arraycopy(rec.getData(),0,signData,0,reclen);
                } else {
                    byte[] newSignData = new byte[signData.length+reclen];
                    System.arraycopy(signData,0,newSignData,0,signData.length);
                    System.arraycopy(rec.getData(),0,newSignData,signData.length,reclen);
                    signData = newSignData;
                }
            } else {
                reclen = -reclen;
                if (textArray == null) {
                    textArray = new byte[reclen];
                    System.arraycopy(rec.getData(),0,textArray,0,reclen);
                } else {
                    byte[] newTextArray = new byte[textArray.length+reclen];
                    System.arraycopy(textArray,0,newTextArray,0,textArray.length);
                    System.arraycopy(rec.getData(),0,newTextArray,textArray.length,reclen);
                    textArray = newTextArray;
                }
            }
            recnum = rec.getLink();
        } while (recnum != DRECEOL);
        if (textArray != null) {
            setText(text+new String(textArray,0));
        }
    }
    
    public boolean matches(String lookup) {
        return (compareTo(lookup) == 0);
    }
    
    public int compareTo(String lookup) {
        if (signData != null) {
            return upperText.compareTo(lookup.toUpperCase());
        }
        String substring = lookup;
        int len = upperText.length();
        if (lookup.length() > len) {
            substring = lookup.substring(0,len);
        }
        int rtn = upperText.compareTo(substring.toUpperCase());
        if (rtn != 0) {
            return rtn;
        }
        loadEntry();
        return upperText.compareTo(lookup.toUpperCase());
    }
    
    public void setText(String text) {
        this.text = text;
        upperText = text.toUpperCase();
    }
    
    public String getText() {
        return text;
    }

    public String makeSignLabel() {
        StringBuffer buf = new StringBuffer();
        DocumentPart[] docParts = makeDocParts();
        for (int i = 0; i < docParts.length; i++) {
            DocumentPart part = docParts[i];
            if (part == null) {
                break;
            }
            if (part == DocumentPart.NEW_LINE) {
                buf.append("\n");
            } else {
                Sign sign = (Sign)part;
                SignFont.add(sign);
                buf.append(sign.getFontIndex());
            }
        }
        return buf.toString().trim();
    }
    
    public DocumentPart[] makeDocParts() {
        if (signData == null) {
            loadEntry();
        }
		DocumentPart[] docParts = new DocumentPart[1];
		int docPartIndex = 0;
		int bufIndex = 0;
		int code = PCNumbers.toUnsigned(signData[bufIndex++]);
		int nextSignPos = 0;
		boolean addedSignComponent = false;
		while(code != SWFile.EOF) {
		    if (docPartIndex >= docParts.length) {
		        DocumentPart[] newarray = new DocumentPart[docParts.length*2];
		        System.arraycopy(docParts,0,newarray,0,docParts.length);
		        docParts = newarray;
		    }
		    if (docPartIndex == 0 && code == SWFile.NEW_LINE) {
                nextSignPos = PCNumbers.toUnsigned(signData[bufIndex++]);
		    } else {
    		    if (code == SWFile.NEW_SIGN) {
    		        if (docPartIndex != 0 || addedSignComponent) {
                        Sign sign = SWFile.getSign(docParts, docPartIndex);
                        sign.setNextSignPos(nextSignPos*2 + SWFile.SP_WIDTH);
                        nextSignPos = PCNumbers.toUnsigned(signData[bufIndex]);
        		        docPartIndex++;
        		    }
        		    bufIndex++;
    		    } else if (code == SWFile.NEW_LINE) {
    		        if (docParts[docPartIndex] != null) {
        		        docPartIndex++;
    		        }
        		    if (docPartIndex >= docParts.length) {
        		        DocumentPart[] newarray = new DocumentPart[docParts.length*2];
        		        System.arraycopy(docParts,0,newarray,0,docParts.length);
        		        docParts = newarray;
        		    }
    		        docParts[docPartIndex++] = DocumentPart.NEW_LINE;
                    nextSignPos = PCNumbers.toUnsigned(signData[bufIndex++]);
    		    } else {
    		        addedSignComponent = true;
                    Sign sign = SWFile.getSign(docParts, docPartIndex);
    		        if (code == SWFile.CHAR_CODE) {
            		    sign.add(new SignCharacter((byte)signData[bufIndex++],(byte)signData[bufIndex++],(byte)signData[bufIndex++]));
        		    } else {
        		        try {
//as long as all dictionaries are created in 4.x, the last
//parameter here should be TRUE:
        		            SignSymbol symbol = new SignSymbol(code,PCNumbers.toUnsigned(signData[bufIndex++]),(byte)signData[bufIndex++],(byte)signData[bufIndex++], true);
                		    sign.add(symbol);
                		} catch (ArrayIndexOutOfBoundsException aioobe) {
                		    aioobe.printStackTrace();
                		    System.err.println("probably arrowhead out of range for now?");
                		}
        		    }
        		}
    		}
		    code = PCNumbers.toUnsigned(signData[bufIndex++]);
		}
        return docParts;
    }
}
