/****************************************************************
 **
 **  $Id: KeyBoxes.java,v 1.17 1998/07/20 00:58:15 ccp Exp $
 **
 **  $Source: /cvs/classes/signwriting/KeyBoxes.java,v $
 **
 ****************************************************************/

package org.signwriting;

import com.dtai.gwt.BorderGadget;
import com.dtai.gwt.GadgetFlowLayout;
import com.dtai.gwt.GWTEventMulticaster;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Event;
import java.awt.Insets;
import java.util.Vector;
import java11.awt.event.AWTEvent;
import java11.awt.event.FocusEvent;
import java11.awt.event.InputEvent;
import java11.awt.event.KeyEvent;
import java11.awt.event.MouseEvent;
import java11.awt.event.MouseListener;
import java11.awt.event.ActionEvent;
import java11.awt.event.ActionListener;

/**
 * KeyBoxes
 * @version 1.1
 * @author DTAI, Incorporated
 */
public class KeyBoxes extends BorderGadget implements MouseListener, ActionListener {

    // states
    private final static int SELECT_SET = 0;
    private final static int SELECT_SYMBOL = 1;
    private final static int SELECT_ROTATION = 2;
    private final static int MAKE_ALTERATIONS = 3;

    private int state;
    private SignDisplay display;
    ActionListener actionListener;

    KeyBox box[] = {
        new KeyBox(),
        new KeyBox(),
        new KeyBox(),
        new KeyBox(),
        new KeyBox(),
        new KeyBox(),
        new KeyBox(),
        new KeyBox(),
        new KeyBox(),
        new KeyBox(),
    };

   /**
     * Constructs a KeyBoxes.
     */
	public KeyBoxes(SignDisplay display) {
	    this.display = display;

	    display.setEditable(true);
	    display.addMouseListener(this);
	    display.setKeyBoxes(this);

	    setBorderType(BorderGadget.ETCHED_IN);
	    setBorderThickness(2);

	    setLayout(new GadgetFlowLayout(GadgetFlowLayout.CENTER,1,1));

        int i = 0;

	    add(box[i++]);
	    add(box[i++]);
	    add(box[i++]);
	    add(box[i++]);
	    add(new KeyBoxSpacer());
	    i = 8;
	    add(box[i++]);
	    add(box[i++]);
	    add(new KeyBoxSpacer());
	    i = 4;
	    add(box[i++]);
	    add(box[i++]);
	    add(box[i++]);
	    add(box[i++]);

	    state = SELECT_SET;

	    for (i = 0; i < box.length; i++) {
	        box[i].addActionListener(this);
	    }
	}
	
	public boolean isActive() {
	    return state != SELECT_SET || display.getParent() != null;
	}

    public void reset() {
        for (int i = 0; i < box.length; i++) {
            box[i].clear();
        }
        state = SELECT_SET;
    }

    public void setSign(Sign sign) {
        reset();
        if (sign == null) {
            display.clear();
	        getOverlayPanel().remove(display);
	    } else {
	        display.setSign((Sign)(sign.clone()));
	        display.showCursor();
	        getOverlayPanel().add(display);
	    }
       // requestFocus();
    }

    public Sign getSign() {
        Sign sign = display.getSign();
        if (sign == null) {
            return null;
        }
        return (Sign)(sign.clone());
    }

    public void setDisplayLocation(Rectangle rect) {
        Dimension size = display.getPreferredSize();
        /*
        if (size.width < rect.width && size.height < rect.height) {
            display.setSize(rect.width, rect.height);
        }*/
        Point p = new Point(rect.x + ((rect.width - size.width) / 2),
                            rect.y + ((rect.height - size.height) / 2));
        //Hardcoded hack to counter hack in SignWriter.editSign() :)
        if (p.x < 0) {
            p.x = 2;
        }
        if (p.y < 0) {
            p.y = 21;
        }
        setDisplayLocation(p);
    }

    public void setDisplayLocation(Point p) {
        display.setLocation(p.x, p.y);
    }

	/**
	 * isFocusTraversable
	 * @return true
	 */
	public boolean isFocusTraversable() {
    	return false;
	}

    /**
     * processKeyEvent
     * @param e	the KeyEvent
     */
    protected void processKeyEvent(KeyEvent e) {
   	    boolean handled = true;
        if (e.getID() == KeyEvent.KEY_PRESSED) {
			if ( !e.isActionKey() ) {
   		        if ( e.isControlDown() ||
					 e.isMetaDown() ) {
					switch ( e.getKeyCode() ) {

						case 3: { // Ctrl-C
							display.copySelectedComponents();
							break;
						}

						case 4: { // Ctrl-D
							display.duplicateSelectedComponents();
							break;
						}

						case 22: { // Ctrl-V
							display.paste();
							break;
						}

						case 24: { // Ctrl-X
							display.cutSelectedComponents();
							break;
						}

						case 26: { // Ctrl-Z
							//line.undo();
							break;
						}

						default: {
						    handled = false;
						}
					}
				}
				else if ( ! e.isMetaDown() ) {
				    if (e.getKeyChar() == ' ') {
        		        getOverlayPanel().remove(display);
				        display.clear();
				    }
				    //Alt-r or Alt-R shrinks the edit box
				    else if ((e.getKeyChar() == 'R' || e.getKeyChar() == 'r') &&
				               ((e.getModifiers() & e.ALT_MASK) == e.ALT_MASK)) {
    		            display.shrink();
				    } 
				    //Alt-e or Alt-E grows the edit box
				    else if ((e.getKeyChar() == 'E' || e.getKeyChar() == 'e') &&
				               ((e.getModifiers() & e.ALT_MASK) == e.ALT_MASK)) {
    		            display.grow();
				    } else {
            			int scanCode = KeyFile.getScanCode(e.getKeyChar());
            			Sign sign = display.getSign();
            			if (sign != null && sign.getSelectedComponentCount() > 0) {
            			    state = MAKE_ALTERATIONS;
    		                display.hideCursor();
            			} else if (state == MAKE_ALTERATIONS) {
            			    state = SELECT_SET;
            			}
            			if (state == SELECT_SET) {
                		    if (scanCode == KeyFile.M) {
        		                display.rotateCursor(e.isShiftDown());
        		            } else {
                                InxOrSymRec rec =
                    			    KeyFile.keyboard[KeyFile.ALPHA_TYPE][scanCode][KeyFile.NO_MODIFIER];
                    			if (rec.noCharVal()) {
                    			    enterKey(scanCode,KeyFile.NO_MODIFIER);
                    			} else {
                    			    SymRec symrec = (SymRec)rec;
                    			    char ch = (char)symrec.codeval;
                    			    enterKey(scanCode,KeyFile.NO_MODIFIER);
                    			}
                    		}
                		} else if (state == SELECT_SYMBOL) {
                		    int keynum = KeyFile.getKeynum(scanCode);
                		    if (keynum >= 0) {
                		        selectSymbol(keynum);
                		    }
                		} else if (state == SELECT_ROTATION) {
                		    int keynum = KeyFile.getKeynum(scanCode);
                		    if (keynum >= 0) {
                    		    selectRotation(keynum);
                		    }
                		} else if (state == MAKE_ALTERATIONS) {
                		    if (scanCode == KeyFile.N) {
                		        incrementFill(e.isShiftDown());
                		    } else if (scanCode == KeyFile.M) {
                		        state = SELECT_SET;
        		                display.rotateCursor(e.isShiftDown());
                		    } else if (scanCode == KeyFile.Comma) {
                		        changeFlop();
                		    } else if (scanCode == KeyFile.Dot) {
                		        incrementVariation(e.isShiftDown());
                		    } else if (scanCode == KeyFile.Slash) {
                		        incrementRotation(e.isShiftDown());
                		    }
                		    display.signChanged();
                		}
            		}
        		}
    		}
    		else { // isActionKey
    			int nextPos = -1;
    			int factor = 1;
    			if (e.isControlDown()) {
    			    factor = 5;
    			}
    			switch( e.getKeyCode() ) {

    		        case KeyEvent.ESCAPE: {
    		            /*
    		            setSign(null);
    		            invokeAction(e);
    		            */
    		            reset();
       		            break;
       		        }
    				case KeyEvent.UP: {
    				    display.processMotion(0,-1*factor);
    					break;
    				}
    				case KeyEvent.LEFT: {
    				    display.processMotion(-1*factor,0);
    					break;
    				}
    				case KeyEvent.DOWN: {
    				    display.processMotion(0,1*factor);
    					break;
    				}
    				case KeyEvent.RIGHT: {
    				    display.processMotion(1*factor,0);
    					break;
    				}
    				case KeyEvent.HOME: {
    					//line.moveTo( 0, e.isShiftDown() );
    					break;
    				}
    				case KeyEvent.END: {
    					//line.moveTo( line.getTextLength(), e.isShiftDown() );
    					break;
    				}
    				case KeyEvent.BACK_SPACE: {
            			display.deleteSelectedComponents();
    					break;
    				}
    				case KeyEvent.DELETE: {
            			display.deleteSelectedComponents();
    					break;
    				}
    				case KeyEvent.ENTER: {
    				    invokeAction(e);
    					break;
    				}
    				default: {
    					handled = false;
    				}
    			}
    		}

		}
		if ( handled ) {
    		e.consume();
    	}

	    super.processKeyEvent( e );
	}

    public void selectSymbol(int keynum) {
        Sign sign = box[keynum].getSign();
        if (sign != null) {
            for (int i = 0; i < box.length; i++) {
                box[i].clear();
            }
            state = SELECT_ROTATION;
            sign.deselectAllComponents();
            if (sign.getComponentCount() > 1) {
    		    display.addSign(sign);
    	        getOverlayPanel().add(display);
    	        getOverlayPanel().setConsumingTransparentClicks(true);
		        SignComponent[] components = sign.getComponents();
		        for (int i = 0; i < components.length; i++) {
        		    components[i].setSelected(true);
        		}
	            state = MAKE_ALTERATIONS;
                display.hideCursor();
            } else {
                SignSymbol sym = (SignSymbol)sign.getComponent(0);
                int numRots = sym.getNumRotations();
	            for (int i = 0; i < box.length; i++) {
	                if (i >= numRots) {
		                box[i].clear();
		            } else {
		                SignSymbol newsym = new SignSymbol(sym);
		                newsym.setRotation(i);
		                int symcode = newsym.getCode();
		                //certiain symbols (namely those in SignSymbol.hash2)
		                //need a hack to rotate them right in the keyboxes.
                        if (SignSymbol.hash2.contains(new Integer(symcode)) &&
                            newsym.getRot() != 0) {
                            newsym.setRot(newsym.getNumRotations() - newsym.getRot());
                        }
	                    box[i].setSignComponent(newsym);
	                }
                }
            }
        }
    }

    public void selectRotation(int keynum) {
        Sign sign = box[keynum].getSign();
        if (sign != null) {
		    display.addSign(sign);
	        getOverlayPanel().add(display);
	        getOverlayPanel().setConsumingTransparentClicks(true);
	        SignComponent[] components = sign.getComponents();
	        for (int i = 0; i < components.length; i++) {
    		    components[i].setSelected(true);
    		}
		    state = MAKE_ALTERATIONS;
            display.hideCursor();
            for (int i = 0; i < box.length; i++) {
                box[i].clear();
            }
        }
    }

	/**
	 * Adds the specified listener to be notified when component
	 * events occur on this component.
	 *
	 * @param l 	the listener to receive the events
	 */
	public synchronized void addActionListener(ActionListener l) {
        actionListener = GWTEventMulticaster.add(actionListener, l);
	}

	/**
	 * Removes the specified listener so it no longer receives
	 * action events on this action.
	 *
	 * @param l 		the listener to remove
	 */
	public synchronized void removeActionListener(ActionListener l) {
        actionListener = GWTEventMulticaster.remove(actionListener, l);
	}

	/**
	 * processEvent
	 *
	 * @param e		a ActionEvent
	 * @return boolean result
	 */
	protected void processEvent(AWTEvent e) {
		if (e instanceof ActionEvent) {
		    processActionEvent((ActionEvent)e);
		} else {
		    super.processEvent(e);
		}
	}

	protected void processActionEvent(ActionEvent e) {
	    if (actionListener != null) {
    		if(e.getID() == ActionEvent.ACTION_PERFORMED) {
    			actionListener.actionPerformed(e);
    		}
    	}
	}

	/**
	 * invokeAction
	 * @param e - TBD
	 */
	public void invokeAction(AWTEvent e) {
	    Event ev = null;
	    if (e != null) {
	        ev = e.getEvent();
	    }
	    ActionEvent action = new ActionEvent( this, ev,
	                               ActionEvent.ACTION_PERFORMED );
	    processEvent( action );
		if ( ! action.isConsumed() ) {
	        invokeDefault(e);
	    }
	}

	private void incrementRotation(boolean clockwise) {
	    Sign sign = display.getSign();
	    if (sign != null) {
    	    SignComponent[] selected = sign.getSelectedComponents();
    	    if (selected.length == 1) {
        	    for (int i = 0; i < selected.length; i++) {
        	        if (selected[i] instanceof SignSymbol) {
            	        ((SignSymbol)selected[i]).incrementRotation(clockwise);
            	    }
        	    }
    		    display.signChanged();
    		}
	    }
    }

	private void incrementFill(boolean clockwise) {
	    Sign sign = display.getSign();
	    if (sign != null) {
    	    SignComponent[] selected = sign.getSelectedComponents();
    	    if (selected.length == 1) {
        	    for (int i = 0; i < selected.length; i++) {
        	        if (selected[i] instanceof SignSymbol) {
            	        ((SignSymbol)selected[i]).incrementFill(clockwise);
            	    }
        	    }
    		    display.signChanged();
    		}
	    }
    }

	private void changeFlop() {
        //for (int i = 0; i < 4; i++)
	        //incrementRotation();
	    Sign sign = display.getSign();
	    if (sign != null) {
    	    SignComponent[] selected = sign.getSelectedComponents();
    	    if (selected.length == 1) {
        	    for (int i = 0; i < selected.length; i++) {
        	        if (selected[i] instanceof SignSymbol) {
            	        ((SignSymbol)selected[i]).changeFlop();
            	    }
        	    }
    		    display.signChanged();
    		}
	    }
    }

	private void incrementVariation(boolean clockwise) {
	    Sign sign = display.getSign();
	    if (sign != null) {
    	    SignComponent[] selected = sign.getSelectedComponents();
    	    if (selected.length == 1) {
        	    for (int i = 0; i < selected.length; i++) {
        	        if (selected[i] instanceof SignSymbol) {
            	        ((SignSymbol)selected[i]).incrementVariation(clockwise);
            	    }
        	    }
    		    display.signChanged();
    		}
	    }
    }

    private int getKeynum(KeyBox keybox) {
        for (int i = 0; i < box.length; i++) {
            if (keybox == box[i]) {
                return i;
            }
        }
        return -1;
    }

    private void enterKey(int scancode,int modifier) {
        if (KeyFile.keyboard[KeyFile.SIGN_TYPE][scancode][KeyFile.NO_MODIFIER] instanceof InxRec) {
            InxRec entry = // may be making an assumption here!
                (InxRec)KeyFile.keyboard[KeyFile.SIGN_TYPE][scancode][KeyFile.NO_MODIFIER];
            int kinx = entry.inx;
    		int nkeys = KeyFile.keymap[kinx++];
    		for (int i = 0; i < box.length; i++) {
    		    if (i >= nkeys) {
    		        box[i].clear();
    		        continue;
    		    }
    	        Sign sign = display.getSign();
    	        if (sign != null) {
                    sign.deselectAllComponents();
    	        }
    		    if (KeyFile.keymap[kinx+1] /*> 127*/ < 0) { // unit symbol
    		        int symcode = PCNumbers.toUnsigned(KeyFile.keymap[kinx++]);
    		        int opt = KeyFile.keyToOpt(PCNumbers.toUnsigned(KeyFile.keymap[kinx++]));
                    SignSymbol ss = new SignSymbol(symcode, opt, (byte)0, (byte)0);
                    //everybody in SignSymbol.hash1 should start out with fill == 0.
                    //some are, but most aren't, so rather than making another subset hashtable,
                    //we just toggle 'em all!!
                    if (SignSymbol.hash1.contains(new Integer(symcode))) {
                        ss.setFill(0);
                    }
                    //everybody but 97 starts out small.  97 starts out medium!!
                    if (symcode == 97) {
                        ss.incrementVariation(false);
                    }
	                //certiain symbols (namely those in SignSymbol.hash2)
	                //need a hack to rotate them right in the keyboxes.
                    if (SignSymbol.hash2.contains(new Integer(symcode)) && 
                        ss.getRot() != 0) {
                        ss.setRot(ss.getNumRotations() - ss.getRot());
                    }

                	box[i].setSignComponent(ss);
            		state = SELECT_SYMBOL;
    		    } else { // compound symbol
            		state = SELECT_SYMBOL;

    		        int sinx = PCNumbers.makeShort(PCNumbers.toUnsigned(KeyFile.keymap[kinx++]),
    		                             PCNumbers.toUnsigned(KeyFile.keymap[kinx++]));
                	int nsyms = KeyFile.keymap[sinx];
                	sinx++;
                	
                	//wow this is ugly
                    int whatDoWeDo = 0;
                    if ((nsyms == 2) && 
                        (PCNumbers.toUnsigned(KeyFile.keymap[sinx]) == 114) &&
                        (PCNumbers.toUnsigned(KeyFile.keymap[sinx + 4]) == 98)) {
                            whatDoWeDo = 1;
                    }
                    if ((nsyms == 5) && 
                        (PCNumbers.toUnsigned(KeyFile.keymap[sinx]) == 114) &&
                        (PCNumbers.toUnsigned(KeyFile.keymap[sinx + 16]) == 180)) {
                            whatDoWeDo = 2;
                    }

    		        box[i].clear();
                	for (int j = 0; j < nsyms; j++) {
                	    int code = PCNumbers.toUnsigned(KeyFile.keymap[sinx]);
                	    int opt = PCNumbers.toUnsigned(KeyFile.keymap[sinx+1]);
                	    int x = KeyFile.keymap[sinx + 3];
                	    int y = KeyFile.keymap[sinx + 2];
                        SignSymbol ss = new SignSymbol(code, opt, (byte)y, (byte)(x/2));
                        
                        //so ugly it hurts...
                        if (code == 114) {
                            ss.setFill(0);
                            if (whatDoWeDo == 1) {
                                ss.incrementVariation(true);
                            }
                            if (whatDoWeDo == 2) {
                                if (ss.getRot() == 1) {
                                    ss.incrementVariation(true);
                                    ss.setFill(1);
                                }
                            }
                        }

                    	box[i].addSignComponent(ss);
                        sinx += 4;
                	}
                }
		    }
		}
    }

	/**
	 * processFocusEvent
	 * @param e		the firing FocusEvent
	 */
	protected void processFocusEvent(FocusEvent e) {
        if (e.getID() == FocusEvent.FOCUS_GAINED) {
    	    setBorderType(BorderGadget.THREED_OUT);
        } else {
    	    setBorderType(BorderGadget.ETCHED_IN);
        }
	    super.processFocusEvent( e );
	}

    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
        if (e.getSource() == display) {
         //   requestFocus();
        }
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() instanceof KeyBox) {
    		if (state == SELECT_SYMBOL) {
		        selectSymbol(getKeynum((KeyBox)e.getSource()));
            } else if (state == SELECT_ROTATION) {
                selectRotation(getKeynum((KeyBox)e.getSource()));
            }
        }
    }
}
