package org.signwriting;

import java.util.*;
import java.io.*;
import java.util.MissingResourceException;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Vector;
import java.awt.Dialog;
import com.dtai.gwt.ButtonGadget;
import com.dtai.gwt.MenuGadget;
import com.dtai.gwt.MenuItemGadget;
import com.dtai.gwt.CheckboxGadget;
import com.dtai.gwt.LabelGadget;
import com.dtai.gwt.Gadget;
import com.dtai.gwt.GadgetShell;

public class Resources extends ResourceBundle {

    private static Resources resources;
    private static Dictionary dictionary;
    private ResourceBundle bundle;
    private SignWriter signWriter;
    private Vector labeledGadgets = new Vector();

    public static void openResources(SignWriter signWriter) {
        String resourceName = "Locale";
        try {
            resources = new Resources(signWriter, new PropertyResourceBundle(
                new BufferedInputStream(
                new FileInputStream(resourceName+".properties"))));
            dictionary = Dictionary.getDictionary();
        } catch (Exception mre) {
            System.err.println("Error reading "+resourceName+".properties");
            mre.printStackTrace();
            System.exit(1);
        }
    }

    public static Resources getResources() {
        return resources;
    }
    
    /**
     * Strip off bad characters to turn a "label" (usually a default
     * button label) into a valid property file resource key
     */
    public static String labelToKey(String label) {
        int len = label.length();
        if (label.endsWith("...")) {
            len -= 3;
            label = label.substring(0,len);
        }
        StringBuffer buf = new StringBuffer(len);
        boolean start = true;
        for (int i = 0; i < len; i++) {
            char ch = label.charAt(i);
            if ((start && Character.isJavaIdentifierStart(ch)) ||
                (!start && Character.isJavaIdentifierPart(ch)) ||
                ch == '.') {
                start = false;
                buf.append(ch);
            }
        }
        return buf.toString();
    }
    
    /**
     * same as getString, but returns the given default if not found
     */
    public String getString(String resourceKey, String defaultString) {
    	try {
    	    return resources.getString(resourceKey);
    	} catch (MissingResourceException mre) {
    	}
    	return defaultString;
    }
        
    public static void apply(String resourceKey, String label, Gadget g) {
        resources.addLabeledGadget(resourceKey, label, g);
        resources.applyResources(resourceKey, label, g);
    }
    
    private void addLabeledGadget(String resourceKey, String label, Gadget g) {
        labeledGadgets.addElement(new LabeledGadget(resourceKey, label, g));
    }
    
    private void applyResources(String resourceKey, String label, Gadget g) {
        int language = signWriter.getLanguage();
        if (language == SignWriter.SIGN_LANGUAGE) {
        	try {
                applyLabel(g,dictionary.getSignLabel(resourceKey));
                g.setFont(SignFont.small);
        	} catch (MissingResourceException mre) {
                language = SignWriter.WORDS;
        	}
        }
        if (language == SignWriter.WORDS) {
            applyLabel(g, resources.getString(resourceKey, label));
            // For MenuItemGadget, I have to set the font back to the default here
            // anyway, because
            // it gets overridden otherwise when the menu is popped up.
            // This is not the best thing to do, but under the circumstances...
            
            // But even so, since I have to set the font back from sign font, let's
            // do it here.
            g.setFont(Gadget.defaultFont);
        }
        if (language == SignWriter.BOTH) {
            String wordLabel = "";
            String signLabel = "";
            try {
                wordLabel = resources.getString(resourceKey, label);
            } catch (MissingResourceException mre) {}
        	try {
        	    signLabel = dictionary.getSignLabel(resourceKey);
        	} catch (MissingResourceException mre) {
                language = SignWriter.WORDS;
        	}
            signLabel += "\n" + wordLabel;
        	try {
                applyLabel(g,signLabel);
                g.setFont(SignFont.small);
        	} catch (MissingResourceException mre) {
                language = SignWriter.WORDS;
        	}
        }
    }
    
    private void applyLabel(Gadget g, String label) {
        if (g instanceof LabelGadget) {
            ((LabelGadget)g).setLabel(label);
        } else if (g instanceof ButtonGadget) {
            ((ButtonGadget)g).setLabel(label);
        } else if (g instanceof MenuGadget) {
            ((MenuGadget)g).setLabel(label);
        } else if (g instanceof MenuItemGadget) {
            ((MenuItemGadget)g).setLabel(label);
        } else if (g instanceof CheckboxGadget) {
            ((CheckboxGadget)g).setLabel(label);
        } else {
            throw new RuntimeException("Gadget type not handled: "+g.getClass().getName());
        }
    }
    
    /**
     * Construct from bundle
     * @param bundle Requests to Resources are delegated to bundle.
     */
    private Resources(final SignWriter signWriter, ResourceBundle bundle) {
        this.signWriter = signWriter;
        this.bundle = bundle;
        signWriter.addUIModeListener(new UIModeListener() {
            public void uiModeChanged(UIModeEvent e) {
                Vector dialogs = new Vector();
        	    LabeledGadget[] lg = new LabeledGadget[labeledGadgets.size()];
        	    labeledGadgets.copyInto(lg);
        	    for (int i = 0; i < lg.length; i++) {
        			lg[i].reapplyLabel();
        			GadgetShell shell = lg[i].g.getShell();
        			if (shell != null) {
            			Dialog d = shell.getDialog();
            			if (d != null) {
                			if (!dialogs.contains(d)) {
                			    dialogs.addElement(d);
                			}
                		}
                	}
        		}
        	    Dialog[] d = new Dialog[dialogs.size()];
        	    dialogs.copyInto(d);
        	    for (int i = 0; i < d.length; i++) {
        			if (d[i].getComponent(0) instanceof GadgetShell) {
            			d[i].getComponent(0).setBackground(signWriter.getWindowColor());
            		    ((GadgetShell)d[i].getComponent(0)).getGadget().setBackground(signWriter.getWindowColor());
            		    //d[i].getComponent(0).setSize(20,20);
            		    //d[i].getComponent(0).validate();
            		    //d[i].getComponent(0).repaint();
            		    ((GadgetShell)d[i].getComponent(0)).getGadget().setSize(20,20);
            		    ((GadgetShell)d[i].getComponent(0)).getGadget().validate();
            		    //((GadgetShell)d[i].getComponent(0)).getGadget().repaint();
        			}
        			d[i].setBackground(signWriter.getWindowColor());
        			d[i].pack();
        			//d[i].invalidate();
        			//d[i].repaint();
        		}
        		dialogs.removeAllElements();
            }
        });
    }

    /** Get an object from a ResourceBundle.
     * <STRONG>NOTE: </STRONG>Subclasses must override.
     * @param key see class description.
     */
    protected Object handleGetObject(String key) throws MissingResourceException {
        return bundle.getObject(key);
    }

    /**
     * Return an enumeration of the keys.
     */
    public Enumeration getKeys() {
        return bundle.getKeys();
    }

    private final class LabeledGadget {
        String resourceKey;
        String label;
        Gadget g;
        LabeledGadget(String resourceKey, String label, Gadget g) {
            this.resourceKey = resourceKey;
            this.label = label;
            this.g = g;
        }
        
        void reapplyLabel() {
            applyResources(resourceKey, label, g);
        }
    }
}
