package org.signwriting;

import com.dtai.gwt.*;
import java.awt.*;

class SignFontMetrics extends SpecialFontMetrics {

    private boolean thumbNail;
    private final int twiceHorizontalMargin;
    private final int verticalMargin;
    private int[] widths;

    SignFontMetrics(SignFont font, boolean thumbNail) {
        super(font);
        this.thumbNail = thumbNail;
        this.twiceHorizontalMargin = font.horizontalMargin * 2;
        this.verticalMargin = font.verticalMargin;
    }
    
    public int getDescent() {
        return super.getDescent();// + verticalMargin;
    }

    /**
     * Returns the total advance width for showing the specified array
     * of characters in this Font.
     * The advance width is the amount by which the current point is
     * moved from one character to the next in a line of text.
     * @param data the array of characters to be measured
     * @param off the start offset of the characters in the array
     * @param len the number of characters to be measured from the array
     * @see #stringWidth
     * @see #bytesWidth
     */
    public int charsWidth(char data[], int off, int len) {
        if (len == 1 && data[off] < SignFont.FONT_INDEX_START) {
            return normalMetrics.charsWidth(data,off,len);
        }
        int totalWidth = 0;
        for (int i = 0; i < len; i++) {
            char ch = data[i+off];
            if (ch >= SignFont.FONT_INDEX_START) {
                Sign sign = ((SignFont)getFont()).getSign(ch);
                if (sign != null) {
                    if (totalWidth > 0) {
                        totalWidth++;
                    }
                    Rectangle bounds = sign.getBounds();
                    if (getFont() == SignFont.small) {
                        bounds.width++;
                        bounds.width /= 2;
                    }
                    totalWidth += bounds.width;
                    if (sign.isWord()) {
                        totalWidth += twiceHorizontalMargin;
                    }
                }
            } else {
                char charArray[] = {ch};
                totalWidth += normalMetrics.charsWidth(charArray,0,1);
            }
        }
    	return totalWidth;
    }

    /**
     * This is specific to SignFont only!
     * Returns the maximum ascent for showing the specified array
     * of characters in this Font.
     * @param data the array of characters to be measured
     * @param off the start offset of the characters in the array
     * @param len the number of characters to be measured from the array
     * @see #stringAscent
     * @see #bytesAscent
     */
    public int charsAscent(char data[], int off, int len) {
        int normAscent = normalMetrics.getAscent();
        if (len == 1 && data[off] < SignFont.FONT_INDEX_START) {
            return normAscent;// + verticalMargin;
        }
        int maxAscent = normAscent;
        for (int i = 0; i < len; i++) {
            char ch = data[i+off];
            if (ch >= SignFont.FONT_INDEX_START) {
                Sign sign = ((SignFont)getFont()).getSign(ch);//(char)(((int)ch)-SignFont.FONT_INDEX_START));
                if (sign != null) {
                    Rectangle bounds = sign.getBounds();
                    if (getFont() == SignFont.small) {
                        bounds.height++;
                        bounds.height /= 2;
                    }
                    if (sign.isWord()) {
                        bounds.height += verticalMargin*2;
                    }
                    maxAscent = Math.max(maxAscent,bounds.height);
                }
            } else {
                maxAscent = Math.max(maxAscent,normAscent);
            }
        }
    	return maxAscent;// + verticalMargin;
    }
    
    
    /**
     * Gets the advance widths of the first 256 OR SO characters in the Font.
     * This returns what the client's getWidths returns.  If that's not
     * 256, so be it.  Check the array length.
     * The advance width is the amount by which the current point is
     * moved from one character to the next in a line of text.
     */
    public int[] getWidths() {
        if (widths == null) {
            widths = new int[256];
            char[] test = new char[1];
            for (int i = 0; i < widths.length; i++) {
                test[0] = (char)i;
                widths[i] = charsWidth(test,0,1);
            }
        }
        int rtn[] = new int[widths.length];
        System.arraycopy(widths, 0, rtn, 0, widths.length);
    	return rtn;
    }
}
