package org.signwriting;

import java.awt.Image;
import java.awt.Color;

public class SignImageEntry {

    private final Image image;
    private final Sign sign;
    private final boolean thumbNail;
    private final Color color;

    SignImageEntry(Image image, Sign sign, boolean thumbNail, Color color) {
        this.image = image;
        this.sign = sign;
        this.thumbNail = thumbNail;
        this.color = color;
        SignImageCache.addEntry(this);
    }
    
    public void dropFromCache() {
        SignImageCache.removeEntry(this);
    }
    
    public final boolean matches(boolean thumbNail, Color color) {
        return ((thumbNail == this.thumbNail) &&
                ((color == this.color) || (color.equals(this.color))));
    }

    public final Image getImage() {
        return image;
    }

    public final Sign getSign() {
        return sign;
    }

    public final boolean isThumbNail() {
        return thumbNail;
    }

    public final Color getColor() {
        return color;
    }

    public boolean equals(Object rhs) {
        if (rhs instanceof SignImageEntry) {
            SignImageEntry entry = (SignImageEntry)rhs;
            return (sign == entry.sign &&
                    thumbNail == entry.thumbNail &&
                    color.equals(entry.color));
        }
        return false;
    }

    public int hashCode() {
        return sign.hashCode() ^
               (thumbNail ? 0x10101010 : 0x01010101) ^
               color.hashCode();
    }
}
